@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -28,7 +28,9 @@ OF_ASSUME_NONNULL_BEGIN
 	id <OHGameControllerProfile> _profile;
 }
 
-- (instancetype)initWithIndex: (int32_t)index type: (uint32_t)type;
+- (instancetype)oh_init OF_UNAVAILABLE;
+- (instancetype)oh_initWithIndex: (int32_t)index
+			    type: (uint32_t)type OF_METHOD_FAMILY(init);
 @end
 
 OF_ASSUME_NONNULL_END
