@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -31,6 +31,7 @@
 #import "OFOptionsParser.h"
 #import "OFSandbox.h"
 #import "OFStdIOStream.h"
+#import "OFSystemInfo.h"
 
 #import "OFArc.h"
 #import "GZIPArchive.h"
@@ -39,6 +40,10 @@
 #import "ZIPArchive.h"
 #import "ZooArchive.h"
 
+#ifdef HAVE_TLS_SUPPORT
+# import "ObjFWTLS.h"
+#endif
+
 #import "OFCreateDirectoryFailedException.h"
 #import "OFGetItemAttributesFailedException.h"
 #import "OFInvalidArgumentException.h"
@@ -49,7 +54,21 @@
 #import "OFSeekFailedException.h"
 #import "OFWriteFailedException.h"
 
-#define bufferSize 4096
+#define bufferSize 16384
+
+#ifdef OF_AMIGAOS
+const char *VER = "$VER: ofarc " OF_PREPROCESSOR_STRINGIFY(OBJFW_VERSION_MAJOR)
+    "." OF_PREPROCESSOR_STRINGIFY(OBJFW_VERSION_MINOR) " (" BUILD_DATE ") "
+    "\xA9 2008-2025 Jonathan Schleifer";
+#endif
+
+#ifdef HAVE_TLS_SUPPORT
+void
+_reference_to_ObjFWTLS(void)
+{
+	_ObjFWTLS_reference = 1;
+}
+#endif
 
 OF_APPLICATION_DELEGATE(OFArc)
 
@@ -72,7 +91,7 @@ help(OFStream *stream, bool full, int status)
 		    @"directory\n"
 		    @"    -E  --encoding=         The encoding used by the "
 		    @"archive\n"
-		    @"                            (only tar, lha and zoo files)"
+		    @"                            (only Tar, LHA and Zoo files)"
 		    @"\n"
 		    @"    -f  --force             Force / overwrite files\n"
 		    @"    -h  --help              Show this help\n"
@@ -89,12 +108,28 @@ help(OFStream *stream, bool full, int status)
 		    @"tgz, zip, zoo)\n"
 		    @"    -v  --verbose           Verbose output for file list"
 		    @"\n"
-		    @"    -x  --extract           Extract files")];
+		    @"    -x  --extract           Extract files\n"
+		    @"        --version           Print the version "
+		    @"information")];
 	}
 
 	[OFApplication terminateWithStatus: status];
 }
 
+static void
+version(void)
+{
+	[OFStdOut writeFormat: @"ofarc %@ (ObjFW %@) "
+			       @"<https://objfw.nil.im/>\n"
+			       @"Copyright (c) 2008-2025 Jonathan Schleifer "
+			       @"<js@nil.im>\n"
+			       @"Licensed under the LGPL 3.0 "
+			       @"<https://www.gnu.org/licenses/lgpl-3.0.html>"
+			       @"\n",
+			       @PACKAGE_VERSION, [OFSystemInfo ObjFWVersion]];
+	[OFApplication terminate];
+}
+
 static void
 mutuallyExclusiveError(OFUnichar shortOption1, OFString *longOption1,
     OFUnichar shortOption2, OFString *longOption2)
@@ -206,21 +241,22 @@ addFiles(id <Archive> archive, OFArray OF_GENERIC(OFString *) *files,
 	OFString *archiveComment, *outputDir, *encodingString, *type;
 	bool isIRI;
 	const OFOptionsParserOption options[] = {
-		{ 'a', @"append", 0, NULL, NULL },
-		{ 0,   @"archive-comment", 1, NULL, &archiveComment },
-		{ 'c', @"create", 0, NULL, NULL },
-		{ 'C', @"directory", 1, NULL, &outputDir },
-		{ 'E', @"encoding", 1, NULL, &encodingString },
-		{ 'f', @"force", 0, NULL, NULL },
-		{ 'h', @"help", 0, NULL, NULL },
-		{ 0,   @"iri", 0, &isIRI, NULL },
-		{ 'l', @"list", 0, NULL, NULL },
-		{ 'n', @"no-clobber", 0, NULL, NULL },
-		{ 'p', @"print", 0, NULL, NULL },
-		{ 'q', @"quiet", 0, NULL, NULL },
-		{ 't', @"type", 1, NULL, &type },
-		{ 'v', @"verbose", 0, NULL, NULL },
-		{ 'x', @"extract", 0, NULL, NULL },
+		{ 'a',  @"append", 0, NULL, NULL },
+		{ '\0', @"archive-comment", 1, NULL, &archiveComment },
+		{ 'c',  @"create", 0, NULL, NULL },
+		{ 'C',  @"directory", 1, NULL, &outputDir },
+		{ 'E',  @"encoding", 1, NULL, &encodingString },
+		{ 'f',  @"force", 0, NULL, NULL },
+		{ 'h',  @"help", 0, NULL, NULL },
+		{ '\0', @"iri", 0, &isIRI, NULL },
+		{ 'l',  @"list", 0, NULL, NULL },
+		{ 'n',  @"no-clobber", 0, NULL, NULL },
+		{ 'p',  @"print", 0, NULL, NULL },
+		{ 'q',  @"quiet", 0, NULL, NULL },
+		{ 't',  @"type", 1, NULL, &type },
+		{ 'v',  @"verbose", 0, NULL, NULL },
+		{ 'x',  @"extract", 0, NULL, NULL },
+		{ '\0', @"version", 0, NULL, NULL },
 		{ '\0', nil, 0, NULL, NULL }
 	};
 	OFUnichar option, mode = '\0';
@@ -249,7 +285,13 @@ addFiles(id <Archive> archive, OFArray OF_GENERIC(OFString *) *files,
 	    [OFIRI fileIRIWithPath: @LOCALIZATION_DIR]];
 #else
 	[OFLocale addLocalizationDirectoryIRI:
-	    [OFIRI fileIRIWithPath: @"PROGDIR:/share/ofarc/localization"]];
+	    [OFIRI fileIRIWithPath: @"PROGDIR:/Data/ofarc/localization"]];
+#endif
+
+#ifdef OF_AMIGAOS
+	[[OFRunLoop mainRunLoop] addExecSignal: SIGBREAKB_CTRL_C
+					target: self
+				      selector: @selector(handleBreakCtrlC:)];
 #endif
 
 	optionsParser = [OFOptionsParser parserWithOptions: options];
@@ -301,6 +343,10 @@ addFiles(id <Archive> archive, OFArray OF_GENERIC(OFString *) *files,
 		case 'h':
 			help(OFStdOut, true, 0);
 			break;
+		case '-':
+			if ([optionsParser.lastLongOption isEqual: @"version"])
+				version();
+			break;
 		case '=':
 			[OFStdErr writeLine: OF_LOCALIZED(
 			    @"option_takes_no_argument",
@@ -487,9 +533,10 @@ addFiles(id <Archive> archive, OFArray OF_GENERIC(OFString *) *files,
 				OFString *attributeName =
 				    @"com.apple.quarantine";
 
-				_quarantine = [[[OFFileManager defaultManager]
+				_quarantine = objc_retain(
+				    [[OFFileManager defaultManager]
 				    extendedAttributeDataForName: attributeName
-						     ofItemAtIRI: IRI] retain];
+						     ofItemAtIRI: IRI]);
 			} @catch (OFGetItemAttributesFailedException *e) {
 				if (e.errNo != /*ENOATTR*/ 93)
 					@throw e;
@@ -712,8 +759,8 @@ error:
 
 	if (_overwrite == -1) {
 		if (_outputLevel >= 0) {
-			[OFStdOut writeString: @" "];
-			[OFStdOut writeLine:
+			[OFStdErr writeString: @" "];
+			[OFStdErr writeLine:
 			    OF_LOCALIZED(@"file_skipped", @"skipped")];
 		}
 		return false;
@@ -726,7 +773,12 @@ error:
 		    @"file", fileName)];
 		[OFStdErr writeString: @" "];
 
-		line = [OFStdIn readLine];
+		if ([OFStdIn hasTerminal])
+			line = [OFStdIn readLine];
+		else {
+			line = @"n";
+			[OFStdErr writeLine: line];
+		}
 
 		if ([line isEqual: @"?"])
 			[OFStdErr writeLine: OF_LOCALIZED(
@@ -745,7 +797,7 @@ error:
 
 	if ([line isEqual: @"n"] || [line isEqual: @"N"]) {
 		if (_outputLevel >= 0)
-			[OFStdOut writeLine: OF_LOCALIZED(@"skipping_file",
+			[OFStdErr writeLine: OF_LOCALIZED(@"skipping_file",
 			    @"Skipping %[file]...",
 			    @"file", fileName)];
 
@@ -753,7 +805,7 @@ error:
 	}
 
 	if (_outputLevel >= 0)
-		[OFStdOut writeString: OF_LOCALIZED(@"extracting_file",
+		[OFStdErr writeString: OF_LOCALIZED(@"extracting_file",
 		    @"Extracting %[file]...",
 		    @"file", fileName)];
 
@@ -770,7 +822,7 @@ error:
 	@try {
 		length = [input readIntoBuffer: buffer length: bufferSize];
 	} @catch (OFReadFailedException *e) {
-		[OFStdOut writeString: @"\r"];
+		[OFStdErr writeString: @"\r"];
 		[OFStdErr writeLine: OF_LOCALIZED(@"failed_to_read_file",
 		    @"Failed to read file %[file]: %[error]",
 		    @"file", fileName,
@@ -781,7 +833,7 @@ error:
 	@try {
 		[output writeBuffer: buffer length: length];
 	} @catch (OFWriteFailedException *e) {
-		[OFStdOut writeString: @"\r"];
+		[OFStdErr writeString: @"\r"];
 		[OFStdErr writeLine: OF_LOCALIZED(@"failed_to_write_file",
 		    @"Failed to write file %[file]: %[error]",
 		    @"file", fileName,
@@ -831,11 +883,11 @@ error:
 		}
 	}
 
-	[path retain];
+	objc_retain(path);
 
 	objc_autoreleasePoolPop(pool);
 
-	return [path autorelease];
+	return objc_autoreleaseReturnValue(path);
 }
 
 - (void)quarantineFile: (OFString *)path
@@ -848,4 +900,23 @@ error:
 				ofItemAtPath: path];
 #endif
 }
+
+- (void)checkForCancellation
+{
+#ifdef OF_AMIGAOS
+	/*
+	 * Perform a single iteration of the run loop to see if we received
+	 * Ctrl-C.
+	 */
+	[[OFRunLoop mainRunLoop] runMode: OFDefaultRunLoopMode
+			      beforeDate: [OFDate distantPast]];
+#endif
+}
+
+#ifdef OF_AMIGAOS
+- (void)handleBreakCtrlC: (ULONG)signal
+{
+	raise(SIGINT);
+}
+#endif
 @end
